; Main Module for TI-92 Expander2 Driver
; Written by Bryan Rittmeyer <bryanr@pair.com>
; Copyright (C) 1997-1999 Horizon Technologies
; http://horizon.pair.com/
;
; All source to this driver is released under the
; terms of the GNU Public License. Please see
; the encoded gpl.txt for more information.
	include "flib.h"
        include "tios.h"
        include "macros.h"
        xdef _main
        xdef _comment

_main:
;        jsr reclib[init]                ; What, like I trust my 68k?!?
; No, this version uses transfer counters -- If they get too high, returns
; to main menu and fixes stack pointer regardless.
; Note to porters -- The older versions used interupts to allow user
; aborts, but this was very messy. See extlink.asm for more info.
; Establish Interupt Vectors
;	move.w #$2700,sr
;	move.l ($100078),old_int
;	move.l #new_int,($100078)
;	move.w #$2000,sr
; Store a copy of the stack
	move.l a7,stack
        
	move.w #1,bar
;        move.w #0,page
;        bsr PutLogo
	bsr ClearScreen
	SetFont #1
	bsr PutScreen                   ; Put Screen & Setup
        bsr PutMenu
        bsr PutReady
	
main_loop:
	tst.w ($75B0)                   ; Has a key been pressed?
	beq main_loop
	move.w ($75B2),d0               ; Get current key
	clr.w ($75B0)
	cmp.w #264,d0                   ; Escape key
	beq exit_prog
	cmp.w #338,d0                   ; Up arrow key
	beq move_up
	cmp.w #344,d0                   ; Down arrow key
	beq move_down
	cmp.w #13,d0
	beq check_bar
        bra main_loop                   ; Check loop again

move_up:
	cmp.w #1,bar
	beq main_loop
	bsr update_bar
	subq.w #1,bar
	bsr update_bar
	bra main_loop

move_down:
        cmp.w #6,bar
	beq main_loop
        bsr update_bar
	addq.w #1,bar
	bsr update_bar
	bra main_loop

update_bar:
;        clr.l d0
        move.w bar,d0
        addq.w #1,d0
        mulu #10,d0
        bsr xor_vid
        rts

check_bar:				; Find out which action to take
	cmp.w #1,bar
	beq list_dir
	cmp.w #2,bar
        beq list_eup
	cmp.w #3,bar
        beq send_file
	cmp.w #4,bar
        beq get_file
        cmp.w #5,bar
        beq my_ti
        cmp.w #6,bar                    ; Format routine
        beq format_eup

; PutLogo:
;       bsr ClearScreen                 ;
;        move.w #3,d0                    ; Coordinates (bytes) d0-x, d1-y
;        move.w #4,d1
;        lea extlogo(pc),a5
;        bsr put_fgs
;        SetFont #0
;;        WriteStr #10,#10,#4,look
;        WriteStr #33,#62,#4,ngim
;        WriteStr #48,#23,#4,web
;        SetFont #1
;        WriteStr #72,#85,#4,prog_name
;        WriteStr #80,#95,#4,beta
;        WriteStr #60,#115,#4,pres
;        bsr wait_key
;        rts

PutScreen:
        WriteStr #9,#5,#4,_comment    ; Setup screen
        WriteStr #120,#5,#4,auth       ; I deserve SOME credit =)
        WriteStr #9,#112,#4,status      ;
        move.w #5,d0
        bsr xor_vid
        move.w #112,d0
        bsr xor_vid
        rts

; Print screen at startup, refresh
PutMenu:
        WriteStr #55,#20,#4,opt1        ;
        WriteStr #55,#30,#4,opt2        ;
        WriteStr #55,#40,#4,opt3        ;
        WriteStr #55,#50,#4,opt4        ;
        WriteStr #55,#60,#4,opt5        ;
        WriteStr #55,#70,#4,opt6
;        move.w #1,bar
        bsr update_bar
        rts                             ; Return

PutReady:
        WriteStr #65,#112,#0,ready     ; Write ready string
	rts

PutPaused:
        WriteStr #65,#112,#0,paused     ; Write ready string
        rts

PutBusy:
        WriteStr #65,#112,#0,busy
	rts

PutTimeo:
	WriteStr #65,#112,#0,timeo	; Write timed out string
	rts

PutTimeohd:	; Timeout if we had to destroy the file handle too
	WriteStr #65,#112,#0,timeohd	; Write timed out string
	rts

; Assorted, required routines

; Interupt Handler -- This processes what happens if user aborts (auto-int 6)
; new_int:
;	move.l stack,a7
;	bra exit_prog

; Time out handler -- What to do when transfer routines time out
to_handle:
	move.l stack,a7		; Correct stack pointer so we don't have
				; an address error on exit
	bsr ClearScreen
	bsr PutScreen           ; Put Screen & Setup
        bsr PutMenu
	cmp.w #1,active_h	; Was this after handle creation?!?
	beq destroy_handle	; Destroy handle if it is unused
        bsr PutTimeo		; Put timeout warning
	bra main_loop

; Destroy the unused handle
destroy_handle:
	move.w table_handle,-(a7)	; Push handle to stack
        jsr tios::HeapFree              ; Destroy leftover handle
	lea 2(a7),a7			; Update stack
	bsr PutTimeohd
	bra main_loop

; Exit program
exit_prog:
;	move.w #$2700,sr
;	move.l old_int,($100078)
;	move.w #$2000,sr
	rts

; Clear screen
ClearScreen:                            ; Clears one bitplane
        movem.l d0/a0-a1,-(a7)          ;
        move.l  #959,d0                 ; The screen is $F00 bytes = 960
        move.l  #LCD_MEM,a1       ; and A1 to the LCD memory (plane 0)
RepClr:
        clr.l   (a1)+                   ; Clear a longword, increase A1 with 4
        dbra    d0,RepClr               ; Repeat
        movem.l (a7)+,d0/a0-a1          ; Pop the registers
        rts

ClearMenu:
        move.w #108,-(a7)       ; Push coords..
        move.w #240,-(a7)       ; Ditto
        move.w #15,-(a7)        ; "
        move.w #0,-(a7)        ; Same deal...
        jsr flib::erase_rect    ; Draw rectangle
	lea 8(a7),a7
        rts

; ClearMenu for file routines
ClearFile:
        move.w #98,-(a7)       ; Push coords..
        move.w #240,-(a7)       ; Ditto
        move.w #15,-(a7)        ; "
        move.w #0,-(a7)        ; Same deal...
        jsr flib::erase_rect    ; Draw rectangle
	lea 8(a7),a7
        rts

;put_fgs:
;        mulu #240,d1
;        add.w d0,d1
;        lea $4440,a6
;        adda d1,a6
;        move.l #649,d7
;        move.b #25,d4
;paint_loop:
;        move.b  (a5)+,(a6)+
;        sub.b #1,d4
;        cmp.b #0,d4
;        bne paint_goon
;        move.b #25,d4
;        add.l #5,a6
;paint_goon:
;        dbf d7,paint_loop
;        rts

; Inverts that portion of the screen
xor_vid:
        mulu #30,d0
        addq.w #1,d0
        move.l #LCD_MEM,a6
        adda.l d0,a6
        move.w #8,d7
        move.b #28,d4
xor_loop:
        eor.b #$FF,(a6)+
        subq.b #1,d4
        cmp.b #0,d4
	beq new_line
	bra xor_loop
new_line:
	subq.b #1,d7
	cmp.b #0,d7
	beq exit_sub
	move.b #28,d4
	addq.l #2,a6
        bra xor_loop

; Delay function from FargoC
delay_loop:
        dbf             d7,delay_loop
        rts

; Pause to get a key
wait_key:
	tst.w ($75B0)                   ; Has a key been pressed?
        beq wait_key
	move.w ($75B2),d0               ; Get current key
	clr.w ($75B0)
        rts

; Other source files
; Boy, this is a very large amount of source.

; TI-92 DIR routines, in seperate source file
        include e2dir.asm

; "My TI-92" routines, in seperate source file
        include myti.asm

; Actual link port routines needed very much by the driver =)
        include e2link.asm

; Hex viewer routines
;        include e2hex.asm

; Filename input and lookup routines
        include e2input.asm

; List files on Extender feature
        include e2list.asm

; Extender file send/receive routines... the real beef ;-)
        include e2file.asm

; Format routines
        include e2form.asm

; VAT file creation routines
        include e2create.asm

; Variable declaration space
;extlogo:
;        INCBIN  "extlogo.bin"

; Text strings for main menu
auth    dc.b    "By Bryan Rittmeyer",0
status  dc.b    "Status:",0
ready   dc.b    "Ready          ",0
paused	dc.b	"Paused         ",0
busy    dc.b    "Busy           ",0
timeo	dc.b	"Timed out!     ",0
timeohd	dc.b	"Timed out! (HD)",0
opt1    dc.b    "List files on TI-92",0
opt2    dc.b    "List files on E2",0
opt3    dc.b    "Send to E2",0
opt4    dc.b    "Receive from E2",0
opt5    dc.b    "My TI-92",0
opt6    dc.b    "Format E2",0
; the     dc.b    ".-=The=-.",0
pres    dc.b    ".-=Press any key=-.",0
;ngim    dc.b    ".-=The Next Generation in Memory Expansion=-.",0
;beta    dc.b    "Private Beta",0
;web     dc.b    "http://horizon.pair.com/e2/",0

; File routines
gfn     dc.b    "Enter file name:",0
fnf     dc.b    "File not found!",0
nospac  dc.b    "No space free on Expander!",0

; Search for EuP strings
; look    dc.b    "Looking for Expander2:",0
; notf    dc.b    "Not found. Go build one, dude!",0
; found   dc.b    "Expander2 found!",0

; Strings for My TI-92 module
memf    dc.b    "Memory Free:",0
runf    dc.b    "Run From   :",0
home    dc.b    "Home",0
shel    dc.b    "Shell",0

; Temporary "trash" word
temp    dc.w    0

; Variable for menu system
bar     dc.w    0

; Declare variables for ROM's sprint routine
format  dc.b    "%ld",0
buffer  dc.b    "        ",0
value   dc.l    0

; Variables for (bloated, needs to be optimized) DIR code
vataddr	dc.l	0
vattemp	dc.l	0
filenum	dc.w	0
count	dc.w	0
pcount	dc.w	0
row	dc.w	0
vatemp	dc.l	0
totsize	dc.l	0

; Define word values for various link port routines
outb    dc.b    0
inb     dc.b    0
page    dc.w    0

; Number of bytes to get && where to start
numba   dc.w    0
num_bk  dc.w    0
pstart  dc.b    0

; Temporary longword
tempvar dc.w    0

; File Types
prgm    dc.b    "Program",0
stng    dc.b    "String",0
txt     dc.b    "Text",0
pict    dc.b    "Picture",0
unkn    dc.b    "Unknown",0

; Handle variables
table_handle    dc.w    0
table_addr      dc.l    0
table_temp      dc.l    0
f_handle        dc.w    0

; Variables for hexdump
; addr	dc.l	0

; bookmark:
;        dc.l    0
;        dc.l    0
;        dc.l    0
;        dc.l    0
;        dc.l    $005340
;        dc.l    $0078B4
;        dc.l    $020000
;        dc.l    $400000

; Variables for input routines
key     dc.w    0
; charpnt dc.w    0
; row     dc.w    0
col     dc.w    0

; Security codes for pages - EUPS, start of file, EUPC, continue of file
secs    dc.b    "EIIS",0
secc    dc.b    "EIIC",0
; Assorted other variables required for routines
type    dc.b    0
size    dc.w    0
filenm  dc.b    "        ",0
buff2   dc.b    "        ",0
fileadr dc.l    0
filesnt dc.b    "File Sent!",0
filerec dc.b    "File Received!",0
version dc.b    "Firmware Revision:",0
firmhlp dc.b    "(EuP =< 4, E2 >= 5)",0
lcount  dc.w    0

; Variables for format routine
warn1   dc.b    "This will erase all data on the",0
warn2   dc.b    "Expander2! Press any key to",0
warn3   dc.b    "abort, or capital F to continue.",0
form    dc.b    "Formatting page:",0
scand   dc.b    "Scanned:",0
noff    dc.b    "No files found.",0
oftot   dc.b    "/ 2047",0
fcount  dc.b    0
; First byte of that page - Used to see if page is free
fbyte   dc.b    0
alexist dc.b    "That file already exists!",0
cantmk  dc.b    "Unable to allocate space!",0
space   dc.b    "        ",0

abortd  dc.b    "Aborted!",0

; Time out variables
stack	dc.l	0
active_h dc.w	0		; handle flag
to_count dc.l	0		; Timeout counter

; ed list vars
oftot2	dc.b	"/ 540672",0

_comment:
        dc.b    "Expand92 v2.00",0
        end
