; Variable Creation Module for Expander2 driver
; Written by Bryan Rittmeyer <bryanr@pair.com>
; Copyright (C) 1997-1999 Horizon Technologies
; http://horizon.pair.com/
;
; Somewhat based on code by Gareth James
; Returns: d0.w=1: Created successfully
; Returns: d0.w=0: Couldn't resize MAIN VAT table (if required - See code)
; Input: table_handle.w - Handle of the file to add
; Input: filenm.b*8: Filename (8 characters, zero terminated/padded)
; Assumes that folder is MAIN and file flags are #0 - Change if needed
; Tested to work!
add_vat:
        bsr find_var_pos        ; How many above bottom to insert at
        move.l vataddr,a0       ; VAT address from previous routine
;        clr.l d1
;        clr.l d2
        move.w (a0)+,d1         ; Number of files before need to resize
        move.w (a0),d2          ; Number of files currently
        cmp.w d1,d2             ; Do we need to resize?
        bne skip_resize
; We have to resize the VAT folder now
;        move.w d2,temp
;        WriteStr #10,#10,#4,resize
;        move.w temp,d2
        mulu #12,d2             ; Number of bytes taken up by entries
;        addq.l #4,d2            ; Two words at top of VAT take space too
        add.l #124,d2           ; 120 bytes for ten future entries
        move.l d2,-(a7)         ; Try to resize to this size
        move.w #$C,-(a7)        ; Handle ID for MAIN folder
        jsr tios::HeapRealloc   ; Try to resize
        lea 6(a7),a7            ; We pushed six bytes
        cmp.w #0,d0             ; We couldn't resize the handle!
        beq exit_sub            ; Exit with d0=0 - Let calling routine handle
        move.l vataddr,a0       ; We made room for 10 more files
        move.w (a0),d1          ; So update the MAIN "max b4 resize" word
        add.w #10,d1            ;
        move.w d1,(a0)+         ; update & Advance to number of files word
        move.w (a0),d2          ; put number of files in d2 again
skip_resize:                    ; We don't need to resize yet
        mulu #12,d2             ; Calculate first free byte in next free entry
        addq.l #4,d2            ;
        move.l vataddr,a0       ;
        adda d2,a0              ; a0 now points to first free byte in VAT
        move.l a0,a1            ; copy a0 to a1
        suba #12,a1             ; a1 now points to last byte of prior entry
        clr.l d4                ; d4 will be our counter
;        move.w #1,d4
; Slides VAT forward making a free entry where we need it
copy_loop:
        cmp.w count,d4          ; Have we bumped all entries down?
        beq make_new
        move.l (a1)+,(a0)+      ; Copy first four bytes down & inc
        move.l (a1)+,(a0)+      ; Copy next four bytes down & inc
        move.l (a1)+,(a0)+      ; Copy last four bytes down & inc
        suba #24,a1             ; Move back
        suba #24,a0             ; 
        addq.w #1,d4            ; Increase our counter by one
        bra copy_loop
make_new:                       ; a1 now points to the first byte of the
;        move.l a1,table_temp
;        WriteStr #10,#20,#4,makenew
;        move.l table_temp,a1
        adda #12,a1             ; We must restore to the last one we sent
        lea filenm(PC),a0       ; VAT entry that we opened up, a0 filename
        move.l (a0)+,(a1)+      ; Copy four of 8 letters, increase pointers
        move.l (a0)+,(a1)+      ; Copy rest of file name
        move.w #0,(a1)+         ; Put in 0 for special file flags
        move.w table_handle,(a1) ; Put table handle into VAT
        move.l vataddr,a0       ; Put VAT address into a0
        adda #2,a0              ; Point to number of files
        move.w (a0),d0          ; Increase number of files by 1
        addq.w #1,d0
        move.w d0,(a0)
        move.w #1,d0            ; We created the file successfully (shew!)
        rts

; vataddr: Address to first byte of main folder
; vattemp : Temporary storage longword
; count   : Counter (return)
; filenum : Number of files in VAT (return)
; Input: filenm(PC) - 8 character file name of file to make
; Returns: number of files that should be below this file when it is
; placed alphabetically in the TI-92 VAT (Total-Count before
; insertion=Number above)
; Works! (Rewritten entirerly by Bryan Rittmeyer)
find_var_pos:
        move.w #0,count
        move.w #$C,d0
        handle_ptr d0,a0
        move.l a0,vataddr
        adda #2,a0
        move.w (a0),filenum
;        clr.l d0
        move.w (a0),d0
        subq.l #1,d0
        mulu #12,d0
        addq.l #4,d0
        move.l vataddr,a0
        adda d0,a0              ; a0 is now end of VAT
        move.l a0,vattemp       ; Put into vattemp

look_loop:
        move.l vattemp,a0
        lea filenm(PC),a1
        bsr cmp_name
        cmp.w #1,d0             ; We've found out where to put it now
        beq exit_sub
        cmp.w #0,d0             ; File already exists!
        beq exit_sub
        sub.l #$0C,vattemp
        addq.w #1,count         ; Move us up one file
        bra look_loop
cmp_name:
        move.l #7,d1
cmp_loop3:
        cmp.b (a0)+,(a1)+       ; Compare the bytes alphabetically
        bgt below_vat           ; This works because of ASCII values
        blt above_vat
        dbra d1,cmp_loop3
        move.w #0,d0
        rts
below_vat:                      ; Filename comes BELOW in the VAT
        move.w #1,d0
        rts
above_vat:                      ; Filename should come ABOVE in VAT
        move.w #2,d0
        rts

; Thanks to Gareth James for information and (broken) source code on which
; this is somewhat based.
