; E2 Communications Module for TI-92 Expander2 Driver
; Written by Bryan Rittmeyer <bryanr@pair.com>
; Copyright (C) 1997-1999 Horizon Technologies
; http://horizon.pair.com/
;
; Begin modular code

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Sets up a page retreiving                                             ;
; Paramters:                                                            ;
; page.w: Page to read bytes from                                       ;
; pstart.b: Starting address within that page                           ;
; numba.w: Number of bytes to get                                       ;
; table_addr.l: Address to place to store incoming data                 ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
get_sfe:
        move.w page,d0          ; Setup page for output (4 leading 0s)
        roxl.w #1,d0
        move.w d0,tempvar
        move.b #$52,outb        ; Read Page Op code
        bsr send_byte
        move.w #500,d7
        bsr delay_loop           ; Allow op code to be processed
        move.b tempvar,outb        ; Send page address
        bsr send_byte
        move.w #500,d7          ; Delay to allow processing
        bsr delay_loop
        move.b tempvar+1,outb      ; Second byte of page address
        bsr send_byte           ; Send second page address
        move.w #500,d7
        bsr delay_loop
        move.b pstart,outb      ; Place to begin (can't do 264 yet!)
        bsr send_byte
        move.w #500,d7          ; Have to send other 32 bits this time
        bsr delay_loop           ; One more byte
        move.w numba,num_bk     ; Backup number for use below
        cmp.w #255,numba        ; We want more than 255 bytes - Go below
        bgt send_tff           ;
        move.b numba+1,outb     ; Tell PIC how many bytes we want (<255)
        bsr send_byte           ;
        move.w #500,d7         ;
        bsr delay_loop           ;
        move.b #128,outb        ; This sets bit 7, telling PIC to skip
        bra init_loop          ; Skip code below dealing with numbers >255
send_tff:
        move.b #255,outb        ; Get 255 on first wrte_pg
        bsr send_byte
        move.w #500,d7
        bsr delay_loop
        sub.w #255,numba        ; Calculate how many we need to get on 
        move.b numba+1,outb     ; SECOND wrte_pg
init_loop:
        bsr send_byte
        move.w #500,d7
        bsr delay_loop
        move.w #0,count         ; Initialize byte counter
        move.l table_addr,table_temp
gp_loop:
        move.w num_bk,d0
        cmp.w count,d0          ; Gotten enough yet?
        beq exit_sub
        clr.b inb
        bsr rec_byte            ; Get a byte... NO TIMEOUT
;        move.b inb,outb
;        bsr disp_byte
        move.l table_temp,a0
        move.b inb,(a0)
        addq.l #1,table_temp
        addq.w #1,count
;        clr.l value
;        move.w count,value+2
;        bsr PrintVal
;        WriteStr #50,#50,#4,buffer
        bra gp_loop

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Paramters:                                                              ;
; page: Page to program                                                   ;
; pstart: Starting address within that page -- Should be 0 always!        ;
; numba: Number of bytes to send -- Should be 264			  ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
send_sfe:
        move.w page,d0          ; Setup page for output (4 leading 0s)
        roxl.w #1,d0
        move.w d0,tempvar
        move.b #$82,outb        ; Read Page Op code
        bsr send_byte
        move.w #100,d7
        bsr delay_loop           ; Allow op code to be processed
        move.b tempvar,outb        ; Send page address
        bsr send_byte
        move.w #100,d7          ; Delay to allow processing
        bsr delay_loop
        move.b tempvar+1,outb      ; Second byte of page address
        bsr send_byte           ; Send second page address
        move.w #100,d7
        bsr delay_loop
        move.b pstart,outb      ; Place to begin (can't do 264 yet!)
        bsr send_byte
        move.w #100,d7          ; Have to send other 32 bits this time
        bsr delay_loop           ; One more byte
        move.w numba,num_bk     ; Backup number for use below
        cmp.w #255,numba        ; We want more than 255 bytes - Go below
        bgt send_tff2           ;
        move.b numba+1,outb     ; Tell PIC how many bytes we want (<255)
        bsr send_byte           ;
        move.w #100,d7         ;
        bsr delay_loop           ;
        move.b #128,outb        ; This sets bit 7, telling PIC to skip
        bra init_loop2          ; Skip code below dealing with numbers >255
send_tff2:
        move.b #255,outb        ; Get 255 on first wrte_pg
        bsr send_byte
        move.w #100,d7
        bsr delay_loop
        sub.w #255,numba        ; Calculate how many we need to get on 
        move.b numba+1,outb     ; SECOND wrte_pg
init_loop2:
        bsr send_byte
        move.w #100,d7
        bsr delay_loop
        rts

; Notice that unlike the receive routine, the send routine has
; a seperately callable put loop. This is because not all of the data
; is in one place! (I.E. you need to be able to program from different
; memory locations)
; Paramters: numba, table_addr
send_page:
        move.w #0,count         ; Initialize byte counter
        move.l table_addr,table_temp
        move.w numba,num_bk
sendp_loop:
        move.w num_bk,d0
        cmp.w count,d0          ; Gotten enough yet?
        beq exit_sub
        move.l table_temp,a0
        move.b (a0),outb
;        bsr disp_byte           ; Display byte we're gonna send
        bsr send_byte           ; Get a byte... NO TIMEOUT
        addq.w #1,count          ; Increase counter by 1
        addq.l #1,table_temp
;        clr.l value
;        move.w count,value+2
;        bsr PrintVal
;        WriteStr #50,#100,#4,buffer
;        move.w #100,d7
;        bsr delay_loop
;        jsr flib[idle_loop]
        tst.w ($75B0)
        beq sendp_loop
        move.w ($75B2),d0
        clr.w ($75B0)
        cmp.w #264,d0
        beq exit_sub
        bra sendp_loop

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Paramters:                                                              ;
; page: Page to erase                                                     ;
; Erases a page of the Serial Flash EEPROM                                ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
erase_sfe:
        move.w page,d0          ; Setup page for output (4 leading 0s)
        roxl.w #1,d0
        move.w d0,tempvar
        move.b #$82,outb        ; Read Page Op code
        bsr send_byte
        move.w #100,d7
        bsr delay_loop           ; Allow op code to be processed
        move.b tempvar,outb        ; Send page address
        bsr send_byte
        move.w #100,d7          ; Delay to allow processing
        bsr delay_loop
        move.b tempvar+1,outb      ; Second byte of page address
        bsr send_byte           ; Send second page address
        move.w #100,d7
        bsr delay_loop
        move.b #0,outb      ; Place to begin (can't do 264 yet!)
        bsr send_byte
        move.w #100,d7          ; Have to send other 32 bits this time
        bsr delay_loop           ; One more byte
        move.b #255,outb     ; Tell PIC how many bytes we want (<255)
        bsr send_byte           ;
        move.w #100,d7         ;
        bsr delay_loop           ;
        move.b #9,outb        ; This sets bit 7, telling PIC to skip
        bsr send_byte
        move.w #100,d7
        bsr delay_loop
        move.w #264,numba
erase_page:
        move.w #0,count         ; Initialize byte counter
; This routine is also seperate from its initializer
erasep_loop:                    ; Also fills in extra page data
        move.w count,d0
        cmp.w numba,d0          ; Gotten enough yet?
        beq exit_sub
        move.b #$FF,outb        ; Write all ones
        bsr send_byte           ; Get a byte... NO TIMEOUT
        addq.w #1,count          ; Increase counter by 1
        tst.w ($75B0)
        beq erasep_loop
        move.w ($75B2),d0
        clr.w ($75B0)
        cmp.w #264,d0
        beq exit_sub
        bra erasep_loop

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Retreive the status register from the EuP                             ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
stat_req:
        move.b #$57,outb        ; Load op code into outb
        bsr send_byte           ;
        bsr rec_byte            ; Get status byte back from PIC
        rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Get firmware version from EuP						;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ver_get:
	move.b #$FF,outb
	bsr send_byte
	bsr rec_byte
	rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Sends a byte, using ROM calls (includes timeout abort)		;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
send_byte:
;        tst.w ($75B0)
;        beq check_port
;        move.w ($75B2),d0
;        clr.w ($75B0)
;        cmp.w #264,d0
;        beq exit_sub
	clr.l to_count
send_check:
	cmp.l #$FFF,to_count	; Have we timed out yet?
	beq to_handle
	addq.l #1,to_count
check_port:
        jsr tios::tx_free     ; Number of bytes free in link port buffer
        cmp.w #0,d0             ;
        beq send_check          ;
        move.w #1,-(a7)         ; Send 1 byte
        pea outb(PC)            ; Push ADDRESS to the byte
        jsr tios::transmit    ; Send the byte
        lea 6(a7),a7            ; Updated Stack
        rts                     ; Return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Wait for and get a byte (or more) using ROM calls                     ;
; Returned: d0 (Number of bytes retreived)                              ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
rec_byte:
;        tst.w ($75B0)
;        beq check_rec
;        move.w ($75B2),d0
;        clr.w ($75B0)
;        cmp.w #264,d0
;        beq exit_prog
	clr.l to_count
rec_check:
	cmp.l #$FFF,to_count
	beq to_handle
	addq.l #1,to_count
check_rec:
        move.w #1,-(a7)         ; Try to receive a byte
        pea inb(PC)             ;
        jsr tios::receive
        lea 6(a7),a7
        cmp.w #0,d0             ; We didn't get a byte, loop until we do
        beq rec_check
        rts                     ; We got a byte... return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Use HEXLIB to display a byte in binary notation                       ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;disp_byte:
;        clr.l d0                ; Print a byte in binary format
;        move.b outb,d0
;        move.l #0,d1
;        move.l #0,d2
;        move.l #7,d4
;        jsr hexlib[put_bin]
;        rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Exit subroutine                                                       ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
exit_sub:
        rts
